package Link;
use strict;

my $WITHOUT_MORA = 0;
my $WITH_MORA = 1;

sub new () {
	my $class = shift;
	my $self = {};
	bless($self,$class);

	my $incomingSegments = shift || "";
	my $incomingMoras    = shift || "";
	my $incomingRoot     = shift || "";

	$self->{indices}   = [];
	$self->{phonemes}  = [];
	$self->{moras}     = [];
	$self->{root}      = [];

	$self->{syllables} = [];
	$self->{feet}      = [];    
	$self->{tonespans} = [];
	$self->{headmoras} = [];
	$self->{LUM} = "";
	$self->{violationVector} = [];

	if ($incomingSegments) {
		my @segArr = split(/,/,$incomingSegments);
		for (@segArr) {
			my ($currPhoneme,$currIndex) = (/^([A-Za-z]+)\((\d+)\)/);
			push(@{$self->{phonemes}},$currPhoneme);
			push(@{$self->{indices}},$currIndex);
		}
	}

	if ($incomingMoras) {
		my @moraArr = split(/,/,$incomingMoras);
		for (@moraArr) {
			my ($currMoraIndex,$currMora) = (/^(\d+)\((.+)\)/);
			my $moraStr = $currMoraIndex . "_mi" . join("i",split(/\+/,$currMora)) . "im";
			push(@{$self->{moras}},$moraStr);
		}
	}

	if ($incomingRoot) {
		my @rootArr = split(/,/,$incomingRoot);
		push(@{$self->{root}},@rootArr);
	}





	$self->{test} = "";

	return $self;
}

sub test      {my $self=shift; if (@_) {$self->{test} =shift;}; return $self->{test};}
sub LUM       {my $self=shift; if (@_) {$self->{LUM}  =shift;}; return $self->{LUM};}


sub violationVector {
	my $self=shift; 
	if (@_) {push (@{$self->{violationVector}},shift)};
	return \$self->{violationVector};
}

sub indices {
	my $self=shift; 
	@{$self->{indices}} = @_  if (@_);
	return $self->{indices};
}

sub phonemes {
	my $self=shift; 
	@{$self->{phonemes}} = @_  if (@_);
	return $self->{phonemes};
}

sub root {
	my $self=shift; 
	@{$self->{root}} = @_  if (@_);
	return $self->{root};
}

sub moras {
	my $self=shift; 
	@{$self->{moras}} = @_  if (@_);
	return $self->{moras};
}

sub syllables {
	my $self=shift; 
	@{$self->{syllables}} = @_ if (@_);
	return $self->{syllables};
}


sub derive {

	my $self = shift;

	my $CONStr  = shift || "";
	my @CON = split(/,/,$CONStr);
	my $CON = \@CON;   # create a reference to an array representing CON

	my @chains;
	my @base;          # the chains used as a base for creating more chains
	my $baseTotal = 0; # how many chains were created in the previous round

	my @epentheticSegments = ("schwa","glot");

	# create first chain
	my $chain = Chain->new();
	$chain->input($self);
	my $faithfulCand = copyLink($chain->input());
	$faithfulCand->syllabify($CON);
	$faithfulCand->evaluate($CON,$chain);   
	$chain->pushLink($faithfulCand);
	push(@chains,$chain);
	push(@base,$chain);

	while (@base) {
		$baseTotal += scalar(@base);
		if ($baseTotal < 20) {
			for my $currChain (@base) {
		
				my $currLink  = $currChain->output();
			
				# deletion
				for (my $segPos=0; $segPos<@{$currLink->indices()}; $segPos++) {
					my $tempLink = copyLink($currLink);
					
					#check to see that deleted segment is not epenthetic
					my $isEpen = 0;
					my $currSegPosIndex = ${$currLink->indices()}[$segPos];
					$isEpen = 1 if (index("@{$currChain->input()->indices()}",$currSegPosIndex)==-1);
					unless ($isEpen) {
						$tempLink->deletion($segPos,$WITH_MORA);
						$tempLink->syllabify($CON);
						$tempLink->evaluate($CON,$currChain);
						push(@chains,$currChain->copyChain($tempLink)) if(isMoreHarmonic($currLink,$tempLink) == 1);
					}
				}

				# mora deletion
				for (my $moraPos=0; $moraPos<@{$currLink->moras()}; $moraPos++) {
					my $tempLink = copyLink($currLink);
					
					#check to see that deleted segment is not epenthetic
					my $isEpen = 0;
					my $currMoraIndex = (${$currLink->moras()}[$moraPos] =~ /^(\d+)_/)[0];
					$isEpen = 1 if (index(" @{$currChain->input()->moras()}", " ".$currMoraIndex."_")==-1);
					unless ($isEpen) {
						$tempLink->moraDeletion($moraPos);
						$tempLink->syllabify($CON);
						$tempLink->evaluate($CON,$currChain);
						push(@chains,$currChain->copyChain($tempLink)) if(isMoreHarmonic($currLink,$tempLink) == 1);
					}
				}

				# change the feature voice
				for (my $segPos=0; $segPos<@{$currLink->indices()}; $segPos++) {
					my $tempLink = copyLink($currLink);
					if (my $newPhone = main::changeFeature(${$tempLink->phonemes()}[$segPos],"voice")) {
						${$tempLink->phonemes()}[$segPos] = $newPhone;
						$tempLink->LUM("Change voice\@" . ${$currLink->indices()}[$segPos]);
						$tempLink->syllabify($CON);
						$tempLink->evaluate($CON,$currChain);
						push(@chains,$currChain->copyChain($tempLink)) if(isMoreHarmonic($currLink,$tempLink) == 1);
					}
				}


				# change the feature high
				for (my $segPos=0; $segPos<@{$currLink->indices()}; $segPos++) {
					my $tempLink = copyLink($currLink);
					if (my $newPhone = main::changeFeature(${$tempLink->phonemes()}[$segPos],"high")) {
						${$tempLink->phonemes()}[$segPos] = $newPhone;
						$tempLink->LUM("Change high\@" . ${$currLink->indices()}[$segPos]);
						$tempLink->syllabify($CON);
						$tempLink->evaluate($CON,$currChain);
						push(@chains,$currChain->copyChain($tempLink)) if(isMoreHarmonic($currLink,$tempLink) == 1);
					}
				}


				# change the feature low
				for (my $segPos=0; $segPos<@{$currLink->indices()}; $segPos++) {
					my $tempLink = copyLink($currLink);
					if (my $newPhone = main::changeFeature(${$tempLink->phonemes()}[$segPos],"low")) {
						${$tempLink->phonemes()}[$segPos] = $newPhone;
						$tempLink->LUM("Change low\@" . ${$currLink->indices()}[$segPos]);
						$tempLink->syllabify($CON);
						$tempLink->evaluate($CON,$currChain);
						push(@chains,$currChain->copyChain($tempLink)) if(isMoreHarmonic($currLink,$tempLink) == 1);
					}
				}


				# change the feature back
				for (my $segPos=0; $segPos<@{$currLink->indices()}; $segPos++) {
					my $tempLink = copyLink($currLink);
					if (my $newPhone = main::changeFeature(${$tempLink->phonemes()}[$segPos],"back")) {
						${$tempLink->phonemes()}[$segPos] = $newPhone;
						$tempLink->LUM("Change back\@" . ${$currLink->indices()}[$segPos]);
						$tempLink->syllabify($CON);
						$tempLink->evaluate($CON,$currChain);
						push(@chains,$currChain->copyChain($tempLink)) if(isMoreHarmonic($currLink,$tempLink) == 1);
					}
				}
			
				# epenthesis of a mora (to some segment)
				for (my $segPos=0; $segPos<@{$currLink->indices()}; $segPos++) {
					my $tempLink = copyLink($currLink);
					$tempLink->moraEpenthesis($segPos,$currChain);
					$tempLink->syllabify($CON);
					$tempLink->evaluate($CON,$currChain);
					push(@chains,$currChain->copyChain($tempLink)) if(isMoreHarmonic($currLink,$tempLink) == 1);
				}

				#epenthesis of a segment (either adding a mora or not)
				for my $epenSeg (@epentheticSegments) {

					# epenthesis of segment + mora
					for (my $segPos=0; $segPos<=@{$currLink->indices()}; $segPos++) {
						my $tempLink = copyLink($currLink);
						$tempLink->epenthesis($epenSeg,$segPos,$currChain,$WITH_MORA);
						$tempLink->syllabify($CON);
						$tempLink->evaluate($CON,$currChain);
						push(@chains,$currChain->copyChain($tempLink)) if(isMoreHarmonic($currLink,$tempLink) == 1);
					}
			
					# epenthesis of just a segment, no mora
					for (my $segPos=0; $segPos<=@{$currLink->indices()}; $segPos++) {
						my $tempLink = copyLink($currLink);
						$tempLink->epenthesis($epenSeg,$segPos,$currChain,$WITHOUT_MORA);
						$tempLink->syllabify($CON);
						$tempLink->evaluate($CON,$currChain);
						push(@chains,$currChain->copyChain($tempLink)) if(isMoreHarmonic($currLink,$tempLink) == 1);
					}
				
				}

				# other repairs?
				
			}
		} else {
			$chains[0]->output()->test(" "); # generate error message in the HTML
		}
		@base = @chains[$baseTotal..$#chains];
	}



	# normalize indices of epenthetic consonants
	for my $currChain (@chains) {
		if(${main::CON()}{F_Dep}->($currChain->output(),$currChain->input())>1) {
			# normalize chains that have more than one Dep violation
			# i.e., that have two or more epenthetic segments

			my $inputLastIndex = 0;
			for(@{$currChain->input()->indices()}) {
				$inputLastIndex = $_ if ($_>$inputLastIndex);
			}

			my $newSegmentIndex = 0;
			for (@{$currChain->output()->indices()}) {
				$newSegmentIndex = $_ if ($_ > $newSegmentIndex) ;
			}
			$newSegmentIndex++;
			
			for my $currIndex (@{$currChain->output()->indices()}) {
				if ($currIndex>$inputLastIndex) {
					for my $currLink (@{$currChain->links()}) {
						my $newLUM = $currLink->LUM();
						$newLUM =~ s/(@|,)${currIndex}/$1${newSegmentIndex}/;
						$currLink->LUM($newLUM);
						$_ =~ s/i${currIndex}i/i${newSegmentIndex}i/ for (@{$currLink->moras()});
						$_ =~ s/i${currIndex}i/i${newSegmentIndex}i/ for (@{$currLink->syllables()});

						for (@{$currLink->indices()}) {
							$_ = $newSegmentIndex if ($_ eq $currIndex);
						}
					}
					$newSegmentIndex++;
				}
			}
		}
	}



	# compute rLUMSeq
	my %outputForms ;
	for (my $i=0; $i<@chains ; $i++) {
		my $outputString = "@{$chains[$i]->output()->phonemes()}@{$chains[$i]->output()->indices()}@{$chains[$i]->output()->syllables()}@{$chains[$i]->output()->moras()}";
		$outputString =~ s/[ \@\(\)]//g;
		$outputForms{$outputString} .= "$i," ; # creates a key of the current form and records the chains that have it

		for(my $j=0 ; $j<$#{$chains[$i]->LUMs()} ; $j++) {
			for(my $k=($j+1) ; $k<@{$chains[$i]->LUMs()} ; $k++) {
				push(@{$chains[$i]->rLUMseq()},"&lt;${$chains[$i]->LUMs()}[$j],${$chains[$i]->LUMs()}[$k]&gt;");
			}
		}

	}
	for (keys %outputForms) {
		my @currChains = split (/,/,$outputForms{$_});
		if (@currChains>1) {
			# positions of chains that have the same output 
			# compare rLUMseq's and reduce them when possible
			for my $a (@currChains) {
				for (my $b=0 ; $b<@{$chains[$a]->rLUMseq()} ; $b++) {
					my $currLUMpair = ${$chains[$a]->rLUMseq()}[$b];
					# for each LUM-pair, count how many chains have it
					my $found = 0;
					for my $c (@currChains) {
						if ($a != $c) {
							for (my $d=0 ; $d<@{$chains[$c]->rLUMseq()} ; $d++) {
								$found++ if(${$chains[$c]->rLUMseq()}[$d] eq $currLUMpair);
							}
						}
					}
					if ($found<$#currChains) {
						# LUM-pair doesn't appear in all relevant chains, must be reduced
						for my $e (@currChains) {
							for (my $f=0 ; $f<@{$chains[$e]->rLUMseq()} ; $f++) {
								splice(@{$chains[$e]->rLUMseq()},$f,1) if (${$chains[$e]->rLUMseq()}[$f] eq $currLUMpair);
							}
						}
					}
				}
			}
		}
	}

	# evaluate PREC constraints
	for (my $i=0; $i<@chains ; $i++) {
		for (my $j=0; $j<@CON ; $j++) {
			if ($CON[$j] =~/^P/) {
				${${$chains[$i]->output()->violationVector()}}[$j] = ${main::CON()}{$CON[$j]}->($chains[$i]);
			}
		}
	}


	
	

	# determine winner(s)
	my @winners = ("0");
	for (my $i=0; $i<@chains; $i++) {
		@winners = ($i) if (isMoreHarmonic($chains[$winners[0]]->output(),$chains[$i]->output()) == 1);
		push(@winners,$i) if (isMoreHarmonic($chains[$winners[0]]->output(),$chains[$i]->output()) == 0);
	}
	$chains[$_]->winner(1) for (@winners);

	return \@chains;
}

sub copyLink {
	my $inLink = shift;
	my $copyAll = shift;

	my $outLink = Link->new();
	
	$outLink->phonemes(@{$inLink->phonemes()});
	$outLink->indices(@{$inLink->indices()});
	$outLink->root(@{$inLink->root()});
	$outLink->moras(@{$inLink->moras()});
	$outLink->LUM($inLink->LUM());

	$outLink->syllables(@{$inLink->syllables()}) if ($copyAll);
	
	return $outLink;
}

sub evaluate {

	my $self = shift;
	my $CON  = shift;
	my $inputChain = shift;
	my $inputForm;
	$inputForm = $inputChain->input() if ($inputChain);

	for (@{$CON}) {
		if (/^P/) {
			$self->violationVector(0); # put zeros for PREC constraints
		} else {
			$self->violationVector(${main::CON()}{$_}->($self,$inputForm));
		}
	}
}


sub isMoreHarmonic {
# this sub takes two Link objects; returns 1 if link2 is more harmonic, -1 if link1 is more harmonic, 0 if equal

	my $link1 = shift;
	my $link2 = shift;
	my $moreHarmonic = 0;
	my @link1violations = @{${$link1->violationVector()}};
	my @link2violations = @{${$link2->violationVector()}};

	for(my $i=0; $i < @link1violations ; $i++) {
		if 	($link1violations[$i] < $link2violations[$i]) {
			$moreHarmonic = -1 ;
			last;
		}
		if 	($link1violations[$i] > $link2violations[$i]) {
			$moreHarmonic = 1 ;
			last;
		}
	}
	return $moreHarmonic;
}


sub countMoras {

	my $self = shift;
	my $segmentPos = shift;

	my $headedMoras = 0;

	if ($segmentPos<@{$self->indices()}) {
		my $currSegIndex = ${$self->indices()}[$segmentPos];
		my $currSegPhoneme = ${$self->phonemes()}[$segmentPos];
	
		for (@{$self->moras()}) {
			if (index($_,"i".$currSegIndex."i")!=-1) {
				my @phonArr = $self->findPhonemes($_); 
				my $maxSonor = 0;
				for my $phon (@phonArr) {
					$maxSonor = main::seg($phon)->sonor() if (main::seg($phon)->sonor() > $maxSonor)
				}
				$headedMoras++ if (main::seg($currSegPhoneme)->sonor() == $maxSonor);
			}
		}
	}
	return $headedMoras;
}


sub findPhonemes {

	my $self = shift;
	my $PCat = shift;
	my @phonArr;

	if (index($PCat,"m")!=-1) {
		$PCat = ($PCat =~ /_m(.+)m$/)[0];
	} else {
		$PCat = ($PCat =~ /_s(.+)s$/)[0];
	}

	my @indexArr = split(/i/,$PCat);
	for (@indexArr) {
		for (my $i=0 ; $i<@{$self->indices()} ; $i++) {
			push(@phonArr,${$self->phonemes()}[$i]) if ($_ eq ${$self->indices()}[$i]);
		}
	}
	return @phonArr;
}





sub syllabify {

	my $self = shift;
	my $CON  = shift;

	my $currentCandidate = copyLink($self);
	$currentCandidate->addSyllables();
	$currentCandidate->evaluate($CON);    # evaluate for markedness only, doesn't get an input form for computing faithfulness

	# Merge syllables
	# Since a syllable was created for each segment that has a mora, some
	# syllables will have to be merged to form codas e.g. a+n -> an (if n has a mora)
	for(my $i=0 ; $i<@{$currentCandidate->syllables()} ; $i++) {
		my $tempCand = copyLink($currentCandidate,1);
		if($tempCand->mergeSyllables($i)) {
			$tempCand->evaluate($CON);
			if(isMoreHarmonic($currentCandidate,$tempCand) == 1) {
				$currentCandidate = $tempCand;
				$i--;
			}
		}
	}

	# adjoining consonants
	my $repeat = 1;
	while ($repeat) {
		# records the syllable strcture in the beginning of the loop
		# will keep looking until there is no change
		my $baseSyllabification = "@{$currentCandidate->syllables()}";

		for(my $i=0 ; $i<@{$currentCandidate->syllables()} ; $i++) {

			# in initial syllable, a good onset is always good
			if ($i==0) {
				my $tempOnset = copyLink($currentCandidate,1);
				if($tempOnset->adjoinOnset($i)){
					$tempOnset->evaluate($CON);
					$currentCandidate = $tempOnset if(isMoreHarmonic($currentCandidate,$tempOnset) == 1);
				}
			}

			if ($i==$#{$currentCandidate->syllables()}) {
				# in final syllable, a good coda is always good
				my $tempCoda = copyLink($currentCandidate,1);
				if ($tempCoda->adjoinCoda($i)) {
					$tempCoda->evaluate($CON);
					$currentCandidate = $tempCoda if (isMoreHarmonic($currentCandidate,$tempCoda) == 1);
				}
			} else {
				# between syllables, compare onset and coda on next syllable
				my $tempNextOnset = copyLink($currentCandidate,1);
				my $tempCoda = copyLink($currentCandidate,1);

				if ($tempNextOnset->adjoinOnset($i+1)) {
					# some consonant was adjoind as an onset
					$tempNextOnset->evaluate($CON);
					if ($tempCoda->adjoinCoda($i)) {
						# some consonant was adjoind as a coda, now choose which one improves harmony more.
						# in a tie, prefer an onset over a coda
						$tempCoda->evaluate($CON);
						if (isMoreHarmonic($currentCandidate,$tempNextOnset)==1 and isMoreHarmonic($currentCandidate,$tempCoda)==1) {
							$currentCandidate = (isMoreHarmonic($tempNextOnset,$tempCoda)==1) ? $tempCoda : $tempNextOnset;
						}
					} else {
						# no consonant was adjoind as a coda
						$currentCandidate = $tempNextOnset if (isMoreHarmonic($currentCandidate,$tempNextOnset)==1);
					}
				} else {
					# no consonant was adjoind as an onset
					if ($tempCoda->adjoinCoda($i)) {
						# some consonant was adjoind as a coda
						$tempCoda->evaluate($CON);
						$currentCandidate = $tempCoda if (isMoreHarmonic($currentCandidate,$tempCoda)==1);
					}
				}
			}
		}
		# stop looping if no change was made in this round
		$repeat=0 if ($baseSyllabification eq "@{$currentCandidate->syllables()}");
	}
	
	$self->syllables(@{$currentCandidate->syllables()});
	$self->moras(@{$currentCandidate->moras()});
}


sub addSyllables {

	my $self = shift;
	my @syllArr;
	
	for (my $i=0 ; $i < @{$self->indices()} ; $i++) {
		my $currSegIndex = ${$self->indices()}[$i];
		my $currSegMoras = $self->countMoras($i);
		push(@syllArr, scalar(@syllArr)+1 . "_si" . $currSegIndex. "is" ) if ($currSegMoras > 0);
	}
	$self->syllables(@syllArr);
}



sub mergeSyllables {

	my $self = shift;      # the form checked for syllable merger
	my $firstSyll = shift; # the position of the first syllable of the two to be merged

	my $change = 0;

	if ($firstSyll < $#{$self->syllables()}) {

		# last segment of first syll
		my $seg1index = (${$self->syllables()}[$firstSyll] =~ /i(\d+)is$/)[0];
		my $seg1pos;
		for (my $i=0 ; $i<@{$self->indices()} ; $i++) {
			$seg1pos = $i if (${$self->indices()}[$i] == $seg1index);
		}

		# is there a segment after the current syllable?
		if (($seg1pos+1)<@{$self->indices()}) {
			my $seg1sonor = main::seg(${$self->phonemes()}[$seg1pos])->sonor();
			my $seg2sonor = main::seg(${$self->phonemes()}[$seg1pos+1])->sonor(); 

			# is the first segment after seg1 inside a syllable?
			my $seg2index   = ${$self->indices()}[$seg1pos+1];
			my $isInSyll;
			$isInSyll = 1 if (index("@{$self->syllables()}","i".$seg2index."i")!=-1);

			my $seg1moras = $self->countMoras($seg1pos);
			my $seg2moras = $self->countMoras($seg1pos+1);
	
			if($seg1moras>0 && $seg2moras==1 && $isInSyll && $seg1sonor>$seg2sonor) {
				# merge syllables
				my $newSyll = (${$self->syllables()}[$firstSyll] =~ /^(.+)s$/)[0] . (${$self->syllables()}[$firstSyll+1] =~ /si(.+)$/)[0];  
				splice (@{$self->syllables()}, $firstSyll, 2, $newSyll);
				$change = 1;
			}
		}
	}
	return $change;
}

sub adjoinOnset {
	
	my $self = shift;
	my $syll = shift;
	my $change = 0;

	if ($syll<@{$self->syllables()}) {
		#first segment of current syllable
		my $seg1index = (${$self->syllables()}[$syll] =~ /si(\d+)i/)[0];
		my $seg1pos;
		for (my $i=0 ; $i<@{$self->indices()} ; $i++) {
			$seg1pos = $i if (${$self->indices()}[$i] eq $seg1index);
		}
		
		# find previous segment, if there is one
		if ($seg1pos>0) {
			my $seg0sonor = main::seg(${$self->phonemes()}[$seg1pos-1])->sonor();
			my $seg1sonor = main::seg(${$self->phonemes()}[$seg1pos])->sonor();
		
			# add seg0 to the syllable if it's less sonorous than the current first segment of the syllable
			if($seg0sonor < $seg1sonor) {
				my $seg0index = ${$self->indices()}[$seg1pos-1];

				# attach seg0 as an onset to current syllable
				${$self->syllables()}[$syll] =~ s/_si/_si${seg0index}i/;
				$change = 1;

				# steal seg0 from its former mora, if it doesn't head its own mora
				# e.g. in kap.ra -> kapa, p must be removed from the first mora 
				if ($self->countMoras($seg1pos-1)==0) {
					for my $currMora (@{$self->moras()}) {
						$currMora =~ s/i${seg0index}im/im/;
					}
				}
			}
		}
	}
	return $change;
}


sub adjoinCoda {

	my $self = shift; # the form that wants a coda
	my $syll = shift; # the position of the syllable that wants a coda
	my $change = 0;   # will indicate whether a consonant was syllabified as a coda

	if ($syll<@{$self->syllables()}) {
		# last segment of $syll
		my $seg1index = (${$self->syllables()}[$syll] =~ /i(\d+)is$/)[0];
		my $seg1pos;
		for (my $i=0 ; $i<@{$self->indices()} ; $i++) {
			$seg1pos = $i if (${$self->indices()}[$i] eq $seg1index);
		}
		
		# Is there a segment after seg1, is it inside a syllable?
		if ($seg1pos+1 < @{$self->indices()}) {

			my $seg1sonor = main::seg(${$self->phonemes()}[$seg1pos])->sonor();
			my $seg2sonor = main::seg(${$self->phonemes()}[$seg1pos+1])->sonor(); 
			my $seg2index = ${$self->indices()}[$seg1pos+1];

			my $isInSyll = 0;
			$isInSyll = 1 if (index("@{$self->syllables()}","i".$seg2index."i")!=-1);
		
			if(($isInSyll==0) and ($seg1sonor>$seg2sonor)) {
	
				# adjoin to syllable
				${$self->syllables()}[$syll] =~ s/is$/i${seg2index}is/;
				$change = 1;
		
				# adjoin to mora
				for (my $i=$#{$self->moras()}; $i>=0; $i--) {
					if (index(${$self->moras()}[$i],"i".$seg1index."i")!=-1)  {
						if (index(${$self->moras()}[$i],"i".$seg2index."i")==-1) {
							${$self->moras()}[$i] =~ s/i${seg1index}im/i${seg1index}i${seg2index}im/;
						}
						last;
					}
				}
			}
		}
	}
	return $change;
}


sub moraDeletion {

	my $self = shift;
	my $position = shift;

	my $delMoraSegIndices = (${$self->moras()}[$position] =~ /\d+_m[i]?(.+)m$/)[0];
	my @delMoraSegIndices = split(/i/,$delMoraSegIndices);
	my @delMoraSegPhonemes = $self->findPhonemes(${$self->moras()}[$position]);

	#find out the index of the mora's head
	my $nucleusSonor = 0;
	my $nucleusLocation = 0;
	for (my $i=0 ; $i<@delMoraSegPhonemes ; $i++) {
		if (main::seg($delMoraSegPhonemes[$i])->sonor() > $nucleusSonor) {
			$nucleusSonor = main::seg($delMoraSegPhonemes[$i])->sonor();
			$nucleusLocation = $i;
		}
	}
	
	splice(@{$self->moras()}, $position , 1);
	$self->LUM("Deletion \@$delMoraSegIndices[$nucleusLocation]")

}



sub deletion {

	
	my $self = shift;
	my $position = shift;
	my $withMoras = shift || 1;
	
	my $delSegIndex   = ${$self->indices()}[$position];
	my $delSegPhoneme = ${$self->phonemes()}[$position];
	my $delSegMoras = $self->countMoras($position);


	# if segments heads moras, and there is deletion with moras, delete moras
	# must do this before deletion of segment, otherwise "findPhonemes" won't find it
	if ($delSegMoras>0 and $withMoras==1) {
		for (my $i=0 ; $i<@{$self->moras()} ; $i++) {
			if (index(${$self->moras()}[$i],"i".$delSegIndex."i")!=-1) {
				my @phonArr = $self->findPhonemes(${$self->moras()}[$i]); 
				my $maxSonor;
				for my $phon (@phonArr) {
					$maxSonor = main::seg($phon)->sonor() if (main::seg($phon)->sonor() > $maxSonor)
				}
				if (main::seg($delSegPhoneme)->sonor() == $maxSonor) {
					splice(@{$self->moras()}, $i , 1);
					$i--;
				}
			}
		}
	}

	# delete appearences of the deleted segment in moras
	for my $currMora (@{$self->moras()}) {
		$currMora =~ s/i${delSegIndex}i/i/;
		$currMora =~ s/mim/mm/;
	}

	# delete segmemt
	splice(@{$self->indices()} ,$position ,1);      # remove element number #position from the array
	splice(@{$self->phonemes()},$position ,1);      # remove element number #position from the array
	$self->LUM("Deletion@" . $delSegIndex);
}


sub moraEpenthesis {

	my $self = shift;
	my $position = shift;
	my $currChain = shift;


	# calculate the index of the new mora
	my $newMoraIndex = 0;
	for (@{$currChain->links()}) {
		for (@{$_->moras()}) {
			my $currMoraIndex = (/^(\d+)_/)[0];
			$newMoraIndex = $currMoraIndex if ($currMoraIndex > $newMoraIndex) ;
		}
	}
	$newMoraIndex++;

	# if mora is epenthesized into another mora, delink following consonants
	my $precMora = -1;
	for (my $i=0 ; $i<$position ; $i++) {
		my $currIndex = ${$self->indices()}[$i] ;
		for (my $j=0 ; $j < @{$self->moras()} ; $j++) {
			$precMora = $j if (index(${$self->moras()}[$j],"i".$currIndex."i")!=-1);
		}
	}
	if (($position-1)>0 and ($position-1)<@{$self->indices()} and $precMora !=-1) {
		my $segmentPrecedingNewIndex = ${$self->indices()}[$position-1];
		${$self->moras()}[$precMora] =~ s/i${segmentPrecedingNewIndex}i.+im$/i${segmentPrecedingNewIndex}im/;
	}

	# add new mora
	my $newMora = $newMoraIndex . "_mi" . ${$self->indices()}[$position] . "im";
	splice(@{$self->moras()},($precMora+1),0,$newMora);

	my $LUM = "Epenthesis \@". ${$self->indices()}[$position];
	$self->LUM($LUM);
}

sub epenthesis {

	my $self = shift;
	my $epenSegment = shift;
	my $position = shift;
	my $currChain = shift;
	my $withMoras = shift;

	# compute index for new segment
	my @segmentIndices;
	for (@{$currChain->links()}) {
		push(@segmentIndices,@{$_->indices()});
	}
	my $newSegmentIndex = 0;
	for my $currIndex (@segmentIndices) {
		$newSegmentIndex = $currIndex if ($currIndex > $newSegmentIndex) ;
	}
	$newSegmentIndex++;


	splice(@{$self->phonemes()},$position,0,$epenSegment);
	splice(@{$self->indices()},$position,0,$newSegmentIndex);

	# if segment is epenthesized into a mora, delink following consonants
	# find such segments
	my $precMora = -1;
	for (my $i=0 ; $i<$position ; $i++) {
		my $currIndex = ${$self->indices()}[$i] ;
		for (my $j=0 ; $j < @{$self->moras()} ; $j++) {
			$precMora = $j if (index(${$self->moras()}[$j],"i".$currIndex."i")!=-1);
		}
	}
	# delink from mora
	if (($position-1)>0 and ($position-1)<@{$self->indices()} and $precMora !=-1) {
		my $segmentPrecedingNewIndex = ${$self->indices()}[$position-1];
		${$self->moras()}[$precMora] =~ s/i${segmentPrecedingNewIndex}i.+im$/i${segmentPrecedingNewIndex}im/;
	}


	# add new mora if required
	if ($withMoras) {
		# calculate the index of the new mora
		my $newMoraIndex = 0;
		for (@{$currChain->links()}) {
			for (@{$_->moras()}) {
				my $currMoraIndex = (/^(\d+)_/)[0];
				$newMoraIndex = $currMoraIndex if ($currMoraIndex > $newMoraIndex) ;
			}
		}
		$newMoraIndex++;

		my $newMora = "${newMoraIndex}_mi${newSegmentIndex}im";
		splice(@{$self->moras()},($precMora+1),0,$newMora);
	}

	my $LUM = "Epenthesis <SPAN CLASS='segment" . main::unicode() . "' STYLE='font-size:120%;'>" . main::seg($epenSegment)->symbol() . "</SPAN>@";

	# look for the first preceding underlying segment
	# if none, add "L", for "L"eft edge
	my $prevInputSegIndex = 0;
	POSITION: while ($position>=0) {
		my $currIndex = ${$self->indices()}[$position];
		for (@{$currChain->input()->indices()}) {
			if ($currIndex == $_) {
				$prevInputSegIndex = $_ ;
				last POSITION;
			}
		}
		$position--;
	}
	$LUM .= ($prevInputSegIndex == 0) ? "L" : $prevInputSegIndex;
	$self->LUM($LUM);
}

sub echoEpenthesis {

	my $self = shift;
	my $originPosition = shift;
	my $newPosition = shift;
	my $currChain = shift;

	my $originalPosIndex = ${$self->indices()}[$originPosition];

	splice(@{$self->phonemes()},$newPosition,0,${$self->phonemes()}[$originPosition]);
	splice(@{$self->indices()} ,$newPosition,0,$originalPosIndex);

	# if segment is epenthesized into a mora, delink following consonants
	# find such segments
	my $precMora = -1;
	for (my $i=0 ; $i<$newPosition ; $i++) {
		my $currIndex = ${$self->indices()}[$i] ;
		for (my $j=0 ; $j < @{$self->moras()} ; $j++) {
			$precMora = $j if (index(${$self->moras()}[$j],"i".$currIndex."i")!=-1);
		}
	}
	# delink from mora
	if (($newPosition-1)>0 and ($newPosition-1)<@{$self->indices()} and $precMora !=-1) {
		my $segmentPrecedingNewIndex = ${$self->indices()}[$newPosition-1];
		${$self->moras()}[$precMora] =~ s/i${segmentPrecedingNewIndex}i.+im$/i${segmentPrecedingNewIndex}im/;
	}

	my $LUM = "Echo-epenthesis($originalPosIndex)@";

	# look for the first preceding underlying segment
	# if none, add "L", for "L"eft edge
	my $prevInputSegIndex = 0;
	POSITION: while ($newPosition>=0) {
		my $currIndex = ${$self->indices()}[$newPosition];
		for (@{$currChain->input()->indices()}) {
			if ($currIndex == $_) {
				$prevInputSegIndex = $_ ;
				last POSITION;
			}
		}
		$newPosition--;
	}
	$LUM .= ($prevInputSegIndex == 0) ? "L" : $prevInputSegIndex;
	$self->LUM($LUM);

}

1