package MakeHTML;
use strict;



sub segmentHTML {

	my $candidate = shift;
	my $segmentPos = shift;
	my $showMora = 1 - (shift || 0);
	my $HTML = shift || 0;

	my $segStr = "" ;

	if ($segmentPos<@{$candidate->indices()}) {
		my $currSegPhoneme = ${$candidate->phonemes()}[$segmentPos];
		my $currSegIndex = ${$candidate->indices()}[$segmentPos];
		
		my $moraCount =  $candidate->countMoras($segmentPos);
		my $moraPostHTML = "" ;
		
		if ((main::seg($currSegPhoneme)->cons() eq "C") and ($moraCount>0) and $showMora) {
			if ($HTML) {
				$moraPostHTML = "<SPAN CLASS='mora'>" . "" x $moraCount ."</SPAN>";
			} else {
				$moraPostHTML = "" x $moraCount;
			}
		};
		
		if (main::seg($currSegPhoneme)->cons() eq "V") {
			$moraPostHTML = (main::unicode()) ? "&#720;" x ($moraCount-1) : "&#249;" x ($moraCount-1)
		};
		
		if ($HTML) {
			$segStr  = "<SPAN CLASS='segment" . main::unicode() . "'>" . main::seg($currSegPhoneme)->symbol() . "$moraPostHTML</SPAN>"  ;
			if ($currSegIndex) { $segStr .=  "<SPAN CLASS='segIndex'>$currSegIndex</SPAN>"; };
		} else {
			$segStr  = main::seg($currSegPhoneme)->symbol() . $moraPostHTML;
		}
	}
 	return $segStr;
}


sub candHTML {

	my $candidate = shift;
	my $HTML = shift || 0;
	my $inChain = shift || 0;
	
	my $candStr = "";

	my $rootFirst = -1;
	my $rootLast = -1;
	my $root = "," . join(",", @{$candidate->root()}) . ",";
	
	for (my $i=0 ; $i<@{$candidate->indices()} ; $i++) {
		$rootFirst = $i if (index($root,",".${$candidate->indices()}[$i].",")!=-1 and $rootFirst==-1);
		$rootLast  = $i if (index($root,",".${$candidate->indices()}[$i].",")!=-1);
	}

	$rootFirst = -1 if ($rootFirst==0);
	$rootLast  = -1 if ($rootLast==$#{$candidate->indices()});
	

	my $processedSegment = 0;

	my %segPosition;
	for (my $i=0 ; $i < @{$candidate->indices()} ; $i++) {
		$segPosition{${$candidate->indices()}[$i]} = $i;
	}

	# spell out syllables and preceding appendices
	for (@{$candidate->syllables()}) {
		my @currSyllIndices = split(/i/,(/_si(.+)is/)[0]);
		my @currSyllPhonemes = $candidate->findPhonemes($_);

		# find where the nucleus is
		my $maxSonor = 0;
		my $maxSonorLocation = 0;
		for (my $i=0 ; $i<@currSyllPhonemes ; $i++) {
			if (main::seg($currSyllPhonemes[$i])->sonor() > $maxSonor) {
				$maxSonor = main::seg($currSyllPhonemes[$i])->sonor();
				$maxSonorLocation = $i;
			}
		}

		# spell out preceding appendix, if any
		if ($processedSegment < $segPosition{$currSyllIndices[0]}) {
			$candStr .= "&lt;";
			for (my $i=$processedSegment ; $i < $segPosition{$currSyllIndices[0]} ; $i++) {
				$candStr .= "[" if($i==$rootFirst);
				$candStr .= segmentHTML($candidate,$i,0,$HTML);
				$candStr .= "]" if($i==$rootLast);
			}
			$candStr .= "&gt;";
		} else {
			# add a period between adjacent syllables
			$candStr .= "." unless ($candStr eq "");
		}
		
		# spell out syllable
		for (my $i=0 ; $i<@currSyllIndices ; $i++) {

			$candStr .= "[" if($segPosition{$currSyllIndices[$i]}==$rootFirst);
			$candStr .= segmentHTML($candidate,$segPosition{$currSyllIndices[$i]},($i==0 and $maxSonorLocation>0),$HTML);
			$candStr .= "]" if($segPosition{$currSyllIndices[$i]}==$rootLast);
		}
		$processedSegment = $segPosition{$currSyllIndices[$#currSyllIndices]}+1;
	}
	
	### spell out any remaining segments after the last syllable,
	### or the whole input if there are no syllables
	if ($processedSegment < @{$candidate->indices()} ) {
		$candStr .= "&lt;" if ($inChain);
		for (my $i=$processedSegment ; $i < @{$candidate->indices()} ; $i++) {
			$candStr .= "[" if($i==$rootFirst);
			$candStr .= segmentHTML($candidate,$i,0,$HTML);
			$candStr .= "]" if($i==$rootLast);
		}
		$candStr .= "&gt;" if ($inChain);
	}


	return $candStr;
}



sub chainHTML {
	
	my $chain = shift;

	# the class "links" hides non-final links in the chain
	my $chainStr = "<SPAN CLASS='links'><SPAN CLASS='chain_delimiter'>&lt;</SPAN>";
	for (my $i=0 ; $i<@{$chain->links()} ; $i++) {
		$chainStr .= "</SPAN>" if ($i==$#{$chain->links()});
		$chainStr .= candHTML(${$chain->links()}[$i],1,1);
		$chainStr .= "<SPAN CLASS='chain_delimiter'> , </SPAN>" if ($i<$#{$chain->links()});
	}
	$chainStr .= "<SPAN CLASS='links'><SPAN CLASS='chain_delimiter'>&gt;</SPAN></SPAN>";
	return $chainStr;

}



sub drawTableau {

	my $input = shift;
	my $CON   = shift;
	my $debug = shift || "";

	my @candidateChains = @{$input->derive($CON)}; # sub "derive" returns a reference to an array of chains

	if ($candidateChains[0]->output->test() eq " ") {
		print "\n<DIV CLASS='pagebody'><SPAN CLASS=errorMsg>This derivation was stopped before it converged due to the large number of candidates that was created.</SPAN><BR> \n You can try again with a ranking that produces less candidates (e.g. with higher ranking faithfulness).</DIV>";
	}

	print "<TABLE CLASS='tableau' BORDER=0 CELLPADDING=0 CELLSPACING=0><TR><TD><TABLE BORDER=0 CLASS='tableau1'  CELLPADDING=6 CELLSPACING=0>\n<TR>\n";
	print "\n\t<TD COLSPAN=3 CLASS='tableau2' STYLE='font-size:130%;line-height:120%;'><SPAN CLASS='chain_delimiter'>/</SPAN>" , candHTML($input,1) , "<SPAN CLASS='chain_delimiter'>/</SPAN></TD>";

	my @CON   = split(/,/,$CON);   # creates an array of constraints
	for(my $i=0 ; $i < @CON ; $i++) {
		print "\n\t<TD ALIGN=center CLASS='tableau2'>" , ${main::CON()}{$CON[$i]}->() , "<BR>" ;
		if ($i>0) {
			print "<A HREF='javascript:rerank($i,-1)'><IMG SRC='images/arrow_ls.gif'  WIDTH=12 HEIGHT=11 HSPACE=3 VSPACE=4 BORDER=0 ALT='promote'></A>";
		} else {
			print                                    "<IMG SRC='images/arrow_lse.gif' WIDTH=12 HEIGHT=11 HSPACE=3 VSPACE=4 BORDER=0 ALT=''>";
		}
		if ($i<$#CON) {
			print "<A HREF='javascript:rerank($i,1)' ><IMG SRC='images/arrow_rs.gif'  WIDTH=12 HEIGHT=11 HSPACE=3 VSPACE=4 BORDER=0 ALT='demote'></A>";
		} else {
			print                                    "<IMG SRC='images/arrow_rse.gif' WIDTH=12 HEIGHT=11 HSPACE=3 VSPACE=4 BORDER=0 ALT=''>";
		}
		print "</TD>";
	}


	for(my $i = 0 ; $i <@candidateChains ; $i++) {
		print "\n<TR>";

		# convert candidate number to letters
		my $singles = $i % 26;
		my $s26 = ($i - $singles)/26 - 1;
		my $number;
		$number = "&#" . scalar(97+$s26) . ";" if ($s26>-1);
		$number .= "&#" . scalar(97+$singles) . ";.";
		print "\n\t<TD CLASS='tableau3'>$number</TD>";

		# print pointing hand for winner
		print ($candidateChains[$i]->winner() ? "\n\t<TD CLASS='tableau3'><IMG SRC='images/hand.gif' width=21 height=10></TD>" : "\n\t<TD CLASS='tableau3'>&nbsp;</TD>"); 

		# print candidate and its info
		print "\n\t<TD CLASS='tableau2'><SPAN STYLE='font-size:130%;line-height:120%;'>" . chainHTML($candidateChains[$i]) . "</SPAN><BR>";

		print "<SPAN CLASS='lums'     >LUMseq: &lt;"  , join(", ",@{$candidateChains[$i]->LUMs()})               , "&gt;<BR></SPAN>" if (@{$candidateChains[$i]->LUMs()});
		print "<SPAN CLASS='rlumseq'  >rLUMseq: \{"   , join(", ",@{$candidateChains[$i]->rLUMseq()})            , "\}<BR></SPAN>"   if (@{$candidateChains[$i]->LUMs()}>1);
		print "<DIV  CLASS='moras'    >Moras: "       , join(",",@{$candidateChains[$i]->output()->moras()})    , "</DIV>" if ($debug);
		print "<DIV  CLASS='syllables'>Syllables: "   , join(",",@{$candidateChains[$i]->output()->syllables()}), "</DIV>" if ($debug);
		print "<SPAN STYLE='color:yellow'>" , $candidateChains[$i]->output()->test() , "</SPAN>";
		print "</TD>";

		# print violation marks
		for(@{${$candidateChains[$i]->output()->violationVector()}}) {
			print "\n\t<TD ALIGN=center CLASS='tableau2'>" ;
			print "&nbsp;" unless ($_);
			($debug)? print $_ : print "*" x $_;
			print "</TD>";
		}
	}
	print "\n</TR>\n</TABLE></TD></TR></TABLE>";
	
	
	
	## output for OTSoft
	#
	#print <<"	EOF";
	#	<A HREF="javascript:toggleView('OTSoft')">OTSoft input</A> &nbsp;
	#	<DIV CLASS="helpDiv" ID="OTSoft">
	#	<BR>
	#	<A HREF="http://www.linguistics.ucla.edu/people/hayes/otsoft/">OTSoft</A> is a freely downloadably program by Bruce Hayes, Bruce Tesar, and Kie Zuraw.<BR>
	#	To create an input text file for use with OTSoft:
	#	<OL>
	#	<LI><A HREF="javascript:document.OTSoft.OTSoft.select()">Select</A> the text in the box below.
	#	<LI>Copy (CTRL+C) the text to the clipboard
	#	<LI>Open a text editor (such as notepad)
	#	<LI>Paste (CTRL+V)
	#	<LI>Don't worry if phonetic symbols don't display correctly.
	#	<LI>Save. In "Encoding", choose "Unicode" rather than "ANSI" to keep your phonetic symbols
	#	<LI>Open the resulting file using OTSoft, or open it with Micro\$oft Excel and choose the correct phonetic font for cells that contain linguistic forms.
	#	<LI>Continue to enjoy as directed by OTSoft.
	#	</OL>
	#	Keep in mind that OTSoft uses classical OT, whereas CCamelOT uses OT-CC. Ranking arguments might be different due to losers that are absent from OT-CC.
	#EOF
	#print "<FORM name='OTSoft'><TEXTAREA NAME='OTSoft'  STYLE='width:500px;height:200px;font-size:100%'>\t\t\t";
	#
	#for(@CON) {
	#	my $currCON = ${main::CON()}{$_}->();
	#	$currCON =~ s/<.+?>//g;
	#	print $currCON, "\t" ;
	#}
	#
	#print "\n\t\t\t";
	#
	#for(@CON) {
	#	my $currCON = ${main::CON()}{$_}->();
	#	$currCON =~ s/<[^>]+>//g;
	#	print $currCON, "\t" ;
	#}
	#
	#print "\n" , candHTML($input,0) , "\t";
	#
	#
	#for(my $i = 0 ; $i <@candidateChains ; $i++) {
	#
	#	print "\t" if $i>0;
	#	
	#	print candHTML($candidateChains[$i]->output()) , "\t";
	#
	#	print $candidateChains[$i]->winner() || "0";
	#
	#	my @currVector = @{${$candidateChains[$i]->output()->violationVector()}};
	#	for(@currVector) {
	#
	#		print "\t", $_ ;
	#	}
	#	print "\n";
	#}
	#
	#print "</TEXTAREA></FORM><BR></DIV>";
	#
	#
	## output for Praat
	#
	#print <<"	EOF";
	#	<A HREF="javascript:toggleView('Praat')">Praat input</A> &nbsp; 
	#	<DIV CLASS="helpDiv" ID="Praat">
	#	<BR>
	#	<A HREF="http://www.fon.hum.uva.nl/praat/">PRAAT</A> is a freely downloadable program by Paul Beorsma and David Weenink.<BR>
	#	To create input text files for use with Praat:
	#	<OL>
	#	<LI><A HREF="javascript:document.Praat.grammar.select()">Select</A> the text in "Grammar" box below.
	#	<LI>Copy (CTRL+C) the text to the clipboard
	#	<LI>Open a text editor (such as notepad)
	#	<LI>Paste (CTRL+V)
	#	<LI>If necessary, manually replace funny looking phonetic symbols (or just don't bother).
	#	<LI>Save (don't save as Unicode, Praat won't be able to open the file)
	#	<LI>If you want Pair distributions in addition to the grammar:<BR> <A HREF="javascript:document.Praat.pairdistrib.select()">Select</A> the text in "Pair distribution" box below.<BR>Copy and paste to a text file as you did for the grammar.
	#	<LI>Continue to enjoy as directed by Praat.
	#	</OL>
	#	Keep in mind that Praat uses classical OT, whereas CCamelOT uses OT-CC. Ranking arguments might be different due to losers that are absent from OT-CC.
	#EOF
	#print "<FORM name='Praat'>Grammar:<BR><TEXTAREA NAME='grammar'  STYLE='width:500px;height:100px;font-size:100%'>";
	#
	#print "\"ooTextFile\"\n\"OTGrammar\"\t";
	#print scalar(@CON) , "\n"; 
	#for(my $i=0; $i<@CON ; $i++) {
	#	my $currCON = ${main::CON()}{$CON[$i]}->();
	#	$currCON =~ s/<SPAN STYLE='font-variant\:small-caps;'>(.+?)<\/SPAN>/\\s\{\U$1\}/g;
	#	$currCON =~ s/<.+?>//g;
	#	print "\"" , $currCON, "\"\t",  (scalar(@CON)-$i)*10 , "\t" ,  (scalar(@CON)-$i)*10    , "\n" ;
	#}
	#print "0\n1\n";
	#
	#
	#print "\"" , candHTML($input,0) , "\"\t" , scalar(@candidateChains) , "\n" ;
	#
	#
	#for(my $i = 0 ; $i <@candidateChains ; $i++) {
	#
	#	print "\t\"" , candHTML($candidateChains[$i]->output()) , "\"";
	#
	#	my @currVector = @{${$candidateChains[$i]->output()->violationVector()}};
	#	for(@currVector) {
	#
	#		print "\t", $_ ;
	#	}
	#	print "\n";
	#}
	#
	#print "</TEXTAREA><BR>";
	#
	#print "Pair distribution:<BR><TEXTAREA NAME='pairdistrib'  STYLE='width:500px;height:100px;font-size:100%'>";
	#
	#print "\"ooTextFile\"\n\"PairDistribution\"\n";
	#print scalar(@candidateChains) , " pairs\n";
	#
	#for(my $i = 0 ; $i <@candidateChains ; $i++) {
	#	print "\"" , candHTML($input,0) , "\"\t\"" , candHTML($candidateChains[$i]->output()) , "\"\t";
	#	($candidateChains[$i]->winner()) ? print "100\n" : print "0\n";
	#}
	#
	#print "</TEXTAREA></FORM><BR></DIV>";
	#



}



1