#!/usr/local/bin/perl -w

use strict;
use warnings;

use CGI;
my $q = new CGI;
use CGI::Cookie;
use URI::Escape;

use Fcntl qw(:flock);

use Phoneme;


# read cookies
my %cookies = fetch CGI::Cookie;

# sets phonetic alphabet
my $alphabet = $cookies{alphabet} || "";
$alphabet = ($alphabet =~ /^alphabet=([^;]*);/)[0] || "";
$alphabet = "APAU" unless ($alphabet =~ /^[IA]PA[U]?$/);
sub unicode { return (index($alphabet,"U")!=-1) ? "U" : ""; }
setAlphabet($alphabet);


# set CON
my $CON = uri_unescape(($cookies{CON} =~ /^CON=([^;]*);/)[0]) || "";

#set inputs
my $inputs = uri_unescape(($cookies{inputs} =~ /^inputs=([^;]*);/)[0]) || "";

# read inputs
my $inputScriptStr;
for (split(/;/,$inputs)) {
    $inputScriptStr .= "        inputs[inputs.length] = \"" . $_ . "\";\n";
}


# read nav bar
my $navStr;
open (NAV, "<navbar.txt") or die "Can't read navbar. $!";
$navStr .= $_ while (<NAV>);
close(NAV);




# create the body of the page
print $q->header;
print <<"EOF";
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<HTML xmlns="http://www.w3.org/1999/xhtml" lang="en-US" xml:lang="en-US">
<HEAD>
	<TITLE>CCamelOT - An OT-CC implementation</TITLE>
	
	<LINK rel="stylesheet" type="text/css" href="mich.css" />
	<META http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />

	<SCRIPT src="Phoneme.js"     type="text/javascript"></SCRIPT>
	<SCRIPT src="inputEditor.js"  type="text/javascript"></SCRIPT>
	
	<SCRIPT type="text/javascript">
		//<![CDATA[

		var inputs = [];
		$inputScriptStr
		
		//]]>
		
		function showTab(elem) {
			var tabs = ["phonology","morphology"];
			var foundElem = false
			for (i in tabs) if (tabs[i]==elem) foundElem =true;
			if(foundElem){
				for (i in tabs) {
					document.getElementById(tabs[i]).style.display = (tabs[i]==elem) ? "block" : "none";
				}
			}
		}

		function init() {
			document.getElementById("input_tab").className = "navOn";
			initApp('$alphabet');
		}
		
	</SCRIPT>
</HEAD>

<BODY LEFTMARGIN=0 TOPMARGIN=0 MARGINWIDTH=0 MARGINHEIGHT=0 onLoad="init();">
<FORM name="inputEditor">

$navStr

<DIV ID="phonology">

	<DIV CLASS="menu">
		<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=4>
		<TR>
			<TD CLASS="menuSpacer"><DIV STYLE="width:28px;">&nbsp;</DIV></TD>
			<TD CLASS="menuOn" ><A HREF='javascript:showTab("phonology")'>Phonology</A></TD>
			<TD CLASS="menuSpacer">&nbsp;</TD>
			<TD CLASS="menuOff"><A HREF='javascript:showTab("morphology")'>Morphology</A></TD>
			<TD CLASS="menuSpacer" WIDTH="100%">&nbsp;</TD>
		</TR>
		</TABLE>
	</DIV>

	<DIV CLASS="menuMain">
		<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0>
		<TR VALIGN="top">
			<TD NOWRAP>
				<DIV STYLE="height:7ex;">
				<INPUT TYPE=button NAME="deleteall" VALUE='clear all'  STYLE="width:5.5em;" CLASS="controlButton" onClick='deleteAll(0)'     TITLE='Start a new input'>
				<INPUT TYPE=button NAME="backspace" VALUE='backspace'  STYLE="width:6.5em;" CLASS="controlButton" onClick='removeSegment(0)' TITLE='Remove the last segment of the input'>
				<INPUT TYPE=button NAME="minusmora" VALUE='-'         STYLE="width:2.5em;" CLASS="controlButton" onClick='removeMora(0)'    TITLE='Remove a mora from the last segments of the input'>
				<INPUT TYPE=button NAME="plusmora"  VALUE='+'         STYLE="width:2.5em;" CLASS="controlButton" onClick='addMora(0)'       TITLE='Add a mora to the last segment of the input'>
				</DIV>
			</TD>
			<TD>
				<DIV ID="newInput" STYLE="margin-left:30px;margin-top:.4ex;"></DIV>
			</TD>
		</TR>
		</TABLE>

		<DIV ID='keyboardVowels' STYLE="position: absolute;left:30em;"></DIV>
		<DIV ID='keyboardConsonants'></DIV>

		<DIV ID="tooltips">&nbsp;</DIV>
	</DIV>
</DIV>
</FORM>


<DIV ID="morphology">

	<DIV CLASS="menu">
		<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=4>
		<TR>
			<TD CLASS="menuSpacer"><DIV STYLE="width:28px;">&nbsp;</DIV></TD>
			<TD CLASS="menuOff"><A HREF='javascript:showTab("phonology")'>Phonology</A></TD>
			<TD CLASS="menuSpacer">&nbsp;</TD>
			<TD CLASS="menuOn" ><A HREF='javascript:showTab("morphology")'>Morphology</A></TD>
			<TD CLASS="menuSpacer" WIDTH="100%">&nbsp;</TD>
		</TR>
		</TABLE>
	</DIV>

	<DIV CLASS="menuMain">
		<FORM name="morphEditor">
		<DIV ID="inputMorph"></DIV>
		</FORM>
	</DIV>
</DIV>

EOF

# read footer
open (FTR, "<footer.txt") or die "Can't read footer. $!";
print while (<FTR>);
close(FTR);

print <<"EOF";

</BODY>
</HTML>

EOF

